import requests
from requests import Response 
from enum import Enum

# HTTP STATUS MESSAGE

GET: str                 = "200: Get"                              # HTTP 200
POST  : str              = "201: Created"                          # HTTP 201
UPDATE: str              = "203: Non-Authoritative Information"    # HTTP 203
DELETE: str              = "204: No Content"                       # HTTP 204
ERROR_BAD_REQUEST: str   = "400: Bad Request"                      # HTTP 400
ERROR_NOT_FOUND: str     = "404: Not Found"                        # HTTP 404


class HTTP_Method(Enum):
    GET     = 0
    POST    = 1
    PUT     = 2
    DELETE  = 3
    
    
def fetch(
    url: str, 
    method: HTTP_Method = HTTP_Method.GET,
    request_headers: dict = {"Content-Type": "application/json"}, 
    request_dict: dict={}) -> Response:
    
    """sumary_line
    
    外部通信を行う:
    引数:
        url -- HTTPリクエストを送信する先の（エンドポイント）URL
        method --  HTTPリクエストメソッド Default: HTTP_Method.GET
        request_headers -- リクエストヘッダー Default:  {"Content-Type": "application/json"}
        request_dict -- dict
    戻り値: 
        Responce
    """
    
    match method:
        case HTTP_Method.GET:
            response = requests.get(url, 
                                    json=request_dict, 
                                    headers=request_headers)
            return response
        
        case HTTP_Method.POST:
            response = requests.post(url, 
                                     json=request_dict, 
                                     headers=request_headers)
            return response
        
        case HTTP_Method.PUT:
            response = requests.put(url, 
                                    json=request_dict, 
                                    headers=request_headers)
            return response
        
        case HTTP_Method.DELETE:
            response = requests.delete(url, 
                                       json=request_dict, 
                                       headers=request_headers)
            return response
        
        case _:
            pass


from playground_common_utils.logger import *
from playground_common_utils.http import *
from flask import jsonify, Response
from typing import Tuple, Any, Union, List

RESPONSE = 0
STATUS_CODE = 1
DATA="data"
MESSAGE="message"

# COMMON MAKE RESPONSE DATA
def response_data(status_code: int = 200, data: Union[dict, List[dict], None] = None, message: str = GET) -> Tuple[Response, int]:
    logger.debug(f"status_code: {status_code}")
    logger.debug(f"data: {data}")
    logger.debug(f"message: {message}")
    
    logger.debug(f"result: {jsonify({DATA: data, MESSAGE: message}), status_code}")
    return jsonify({DATA: data, MESSAGE: message}), status_code


def dict_to_model(request_dict: dict, model: Any) -> Any:
    logger.debug(f"request_dict: {request_dict}")
    
    # request_data(type: dict)からkey, value抽出
    for key, value in request_dict.items():
        
        # modelに追加
        setattr(model, key, value)
    
    logger.debug(f"model: {model}")
    return model


def model_to_dict(model: Any, exclude: List[str] = []) -> dict:
    logger.debug(f"model: {model}")
    result = {}
    for column in model.__table__.columns:
        if column.name in exclude:
            continue
        value = getattr(model, column.name)
        if hasattr(value, 'isoformat'):  # datetimeなど
            value = value.isoformat()
        result[column.name] = value
    
    logger.debug(f"dict: {result}")
    return result

def models_to_dicts(model_list: List[Any], exclude: List[str] = []) -> List[dict]:
    return [model_to_dict(model, exclude) for model in model_list]



import logging

class CustomColorEmojiFormatter(logging.Formatter):
    COLORS = {
        logging.DEBUG: "\033[90m",   # 明るい黒 (グレー)
        logging.INFO: "\033[36m",    # シアン
        logging.WARNING: "\033[33m", # 黄色
        logging.ERROR: "\033[31m",   # 赤
        logging.CRITICAL: "\033[91m",# 明るい赤 (より強調)
    }
    RESET_COLOR = "\033[0m"

    EMOJIS = {
        logging.DEBUG: "🐞",
        logging.INFO: "ℹ️ ",
        logging.WARNING: "⚠️ ",
        logging.ERROR: "🚨",
        logging.CRITICAL: "🔥",
    }

    # フォーマッターの初期化時に基本となるフォーマット文字列を受け取る
    def __init__(self, fmt=None, datefmt=None, style='%'):
        super().__init__(fmt, datefmt, style)

    def format(self, record):
        # まず、親クラスのformatメソッドを呼び出して基本的なメッセージをフォーマットする
        # この時点では、まだ色や絵文字は付いていない
        original_message = super().format(record)

        # ログレベルに応じた色と絵文字を取得
        color_code = self.COLORS.get(record.levelno, "") # 色がなければ空文字列
        emoji = self.EMOJIS.get(record.levelno, "")      # 絵文字がなければ空文字列

        # 絵文字と色を元のメッセージに結合する
        # ここで絵文字とカラーコードを挿入し、最後に色をリセットする
        # 絵文字とカラーコードの間にスペースを入れることで、見た目を整える
        colored_message = f"{color_code}{emoji} {original_message}{self.RESET_COLOR}"
        
        return colored_message


# ロガーの設定
logger = logging.getLogger(__name__)


# ログレベル
DEBUG = logging.DEBUG
INFO = logging.INFO
WARNING = logging.WARNING
ERROR = logging.ERROR
CRITICAL = logging.CRITICAL

logger.setLevel(DEBUG)


# ハンドラーの設定
handler = logging.StreamHandler()
# フォーマッターの初期化時に、logging.basicConfigで使ったような一般的なフォーマット文字列を渡す
# 絵文字と色はこのフォーマッターのformatメソッド内で追加される
formatter = CustomColorEmojiFormatter(
    fmt='%(asctime)s [%(levelname)s] | %(filename)s / %(funcName)s  [%(lineno)s] | %(message)s',
    datefmt='%Y-%m-%d %H:%M:%S'
)
handler.setFormatter(formatter)
logger.addHandler(handler)




# setup.py

from setuptools import setup, find_packages

setup(
    name='playground-common-utils',          # パッケージ名
    version='1.4.0',            # バージョン
    packages=find_packages(),  # packagesに自動でplayground_common_utils/が入る
    install_requires=["requests", 
                      "flask"],        # 外部ライブラリ
    author='Hiroki Umatani',        
    author_email='h.umatani@playground.style',
    description='汎用ユーティリティ（ファイル操作、ログ出力など）',
    long_description=open('README.md').read(),  
    long_description_content_type='text/markdown',
    url='https://github.com/HirokiUmatani/playground-common-utils',
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License', 
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.10',
)



import os
import shutil
from enum import Enum
from playground_common_utils.logger import logger


###########
### 確認 ###
###########
def is_exist(path: str) -> bool:
    """
    存在チェック
    引数:
        path -- 存在チェックをするパス<str>
    戻り値: 
        存在有無<bool>
    """
    return os.path.exists(path)

def is_file(path: str) -> bool:
    """
    ファイルの存在チェック
    引数:
        path -- 存在チェックをするパス<str>
    戻り値: 
        存在有無<bool>
    """
    return os.path.isfile(path)

def is_dir(path: str) -> bool:
    """
    ディレクトリの存在チェック
    引数:
        path -- 存在チェックをするパス<str>
    戻り値: 
        存在有無<bool>
    """
    return os.path.isdir(path)

def is_match_first_keyword(target: str, first_keyword: str) -> bool:
    """
    キーワードで始まる文字の判定
    引数:
        target -- 検索対象の文字列r<str>
        first_keyword -- 文字列先頭のキーワード<str>
    戻り値: 
        検索一致<bool>
    """
    return target.endswith(first_keyword)

def is_match_last_keyword(target: str, last_keyword: str) -> bool:
    """
    キーワードで終わる文字の判定
    引数:
        target -- 検索対象の文字列r<str>
        last_keyword -- 文字列最後のキーワード<str>
    戻り値: 
        検索一致<bool>
    """
    return target.endswith(last_keyword)
    
###########
### 作成 ###
###########
def create_file(path: str):
    """
    空ファイルを作成する
    （すでに存在している場合は何もしない）
    
    引数:
        path -- 作成するファイルのパス<str>
        mode -- [Default]READ（読み込み）・WRITE（書き込み）・ADD（追記） 
    戻り値:
        なし
    """
    if not is_exist(path):
        with open(path, 'a', encoding='utf-8'): 
            pass
        logger.debug(f"空ファイル作成: {path}")
    else:
        logger.warning(f"ファイル既に存在: {path}")

def create_dir(path: str):
    """
    ディレクトリの生成
    引数:
        path -- ディレクトリを生成するパス<str>
    戻り値: 
        存在有無<bool>
    """
    if is_dir(path):
        logger.debug(f"ディレクトリ既に存在: {path}")
        return 
    os.mkdir(path)
    logger.info("ディレクトリを生成しました。")


###########
### 取得 ###
###########
def read_current_dir():
    """
    現在のディレクトリ取得
    引数:
        なし
    戻り値:
        なし
    """
    return os.getcwd()

def read_file(path: str):
    """
    ファイルの読み込み
    引数:
        path -- ファイルのパス<str>
    戻り値: 
        ファイルの中身<str>
    """
    with open(path, mode='r', encoding='utf-8') as f:
        return f.read()

def read_list_dir(path: str):
    """
    ディレクトリ内の一覧を取得する

    引数:
        path -- 対象ディレクトリのパス<str>
    戻り値:
        ディレクトリ内のファイル・ディレクトリ一覧<list>
    """
    if is_dir(path):
        items = os.listdir(path)
        logger.debug(f"ディレクトリ一覧取得完了: {path}")
        return items
    else:
        logger.warning(f"対象ディレクトリが存在しません: {path}")
        return []

def read_enviroment(key: str):
    """
    環境変数の値を取得

    引数:
        key -- 環境変数のキー<str>
    戻り値:
        環境変数の値<str>
    """
    return os.environ.get(key)

###########
### 更新 ###
###########
def overwrite_file(path: str, content: str):
    """
    ファイルに内容を書き込む（上書き）
    引数:
        path -- ファイルのパス<str>
        content -- 新たに書き込む文字列<str>
    戻り値:
        なし
    """
    with open(path, mode='w', encoding='utf-8') as f:
        f.write(content)
        logger.debug(f"上書き完了: {path}")

def append_file(path: str, content: str):
    """
    ファイルに内容を追記する
    
    引数:
        path -- ファイルのパス<str>
        content -- 追記する文字列<str>
    戻り値:
        なし
    """
    with open(path, mode='a', encoding='utf-8') as f:
        f.write(content)
        logger.debug(f"追記完了: {path}")

def load_env_file(path: str):
    """
    .envファイルを読み込んで環境変数にセットする

    引数:
        path -- .envファイルのパス<str>
    戻り値:
        なし
    """
    if not is_file(path):
        logger.warning(f".envファイルが存在しません: {path}")
        return

    with open(path, mode='r', encoding='utf-8') as f:
        lines = f.readlines()

    for line in lines:
        line = line.strip()
        if line and not line.startswith("#"):  # 空行とコメントは無視
            key_value = line.split('=', 1)
            if len(key_value) == 2:
                key, value = key_value
                os.environ[key.strip()] = value.strip()
                logger.debug(f"環境変数設定: {key.strip()}")
            else:
                logger.debug(f"無効な行: {line}")


###########
### 削除 ###
###########
def delete_file(path: str):
    """
    ファイルを削除する

    引数:
        path -- 削除するファイルのパス<str>
    戻り値:
        なし
    """
    if is_exist(path):
        os.remove(path)
        logger.debug(f"ファイル削除完了: {path}")
    else:
        logger.debug(f"ファイルが存在しないか、ファイルではない: {path}")

def delete_dir(path: str):
    """
    ディレクトリの削除
    引数:
        path -- ディレクトリを削除するパス<str>
    戻り値: 
        存在有無<bool>
    """
    if is_dir(path):
        os.rmdir(path)
        logger.info("ディレクトリを削除しました。")
    else:
        logger.debug(f"ディレクトリ既に存在: {path}")


###########
### 操作 ###
###########
def copy_file(src_path: str, dst_path: str):
    """
    ファイルをコピーする

    引数:
        src_path -- コピー元のパス<str>
        dst_path -- コピー先のパス<str>
    戻り値:
        なし
    """
    if is_file(src_path):
        shutil.copy(src_path, dst_path)
        logger.debug(f"ファイルコピー完了: {src_path} → {dst_path}")
    else:
        logger.warning(f"コピー元ファイルが存在しません: {src_path}")

def move_file(src_path: str, dst_path: str):
    """
    ファイルを移動する

    引数:
        src_path -- 移動元のパス<str>
        dst_path -- 移動先のパス<str>
    戻り値:
        なし
    """
    if is_file(src_path):
        shutil.move(src_path, dst_path)
        logger.debug(f"ファイル移動完了: {src_path} → {dst_path}")
    else:
        logger.warning(f"移動元ファイルが存在しません: {src_path}")
